unit ReportMaker;
// Component: TReportMaker 1.2
// Author: Johnny Mamenko (c) 1999
// e-mail: mamenko@iname.com
// http://attend.to/johnny

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, db, ComObj;

type
  TReportType = (rtReport, rtCard);

  TPercentCompleteEvent = procedure (Sender : TObject; PercentComplete : integer;
                                     var ContinueProcess : boolean) of object;

  TReportMaker = class(TComponent)
  private
    { Private declarations }
    FRow, FCol : Integer;
    FTemplateFileName, FResultFileName, FPrefixChar : string;
    FReportType : TReportType;
    FPercentComplete : TPercentCompleteEvent;
    FDataSet : TDataSet;
  protected
    { Protected declarations }
    procedure DoPercentComplete(PercentComplete : integer; var ContinueProcess : boolean); virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetDim(Index : integer; Value: integer);
  public
    { Public declarations }
    procedure Execute;
    constructor Create(AOwner : TComponent); override;
  published
    { Published declarations }
    property Row : integer index 0 read FRow write SetDim default 1;
    property Col : integer index 1 read FCol write SetDim default 1;
    property TemplateFileName : string read FTemplateFileName write FTemplateFileName;
    property ResultFileName : string read FResultFileName write FResultFileName;
    property PrefixChar : string read FPrefixChar write FPrefixChar;
    property ReportType : TReportType read FReportType write FReportType;
    property OnPercentComplete : TPercentCompleteEvent read FPercentComplete write FPercentComplete;
    property DataSet : TDataSet read FDataSet write FDataSet;
  end;

procedure Register;

implementation

var
  App : OleVariant;

constructor TReportMaker.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Row:=1;
  Col:=1;
  FDataSet:=nil;
  FResultFileName:='result';
  FPrefixChar:='@';
end;//Create

procedure TReportMaker.SetDim(Index : integer; Value: integer);
begin
  case Index of
    0: if Value<>FRow then
        begin
          if Value<1 then exit;
          FRow:=Value;
        end;
    1: if Value<>FCol then
        begin
          if Value<1 then exit;
          FCol:=Value;
        end;
    end;
end;//SetDim for Col&Row

procedure TReportMaker.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation=opRemove) and (AComponent=FDataset)
    then FDataSet:=nil;
end;//Notification

procedure TReportMaker.DoPercentComplete(PercentComplete : integer; var ContinueProcess : boolean);
begin
  if Assigned(FPercentComplete) then
    FPercentComplete(self, PercentComplete, ContinueProcess);
end;//DoPercentComplete

procedure TReportMaker.Execute;
var c, r, i, j, p, rr: integer;
   cont : boolean; S,s1: String;
begin
  if FReportType=rtReport then
    begin
      if not DataSet.Active then
        begin
          MessageDlg('   ',mtError,[mbOk],0);
          exit;
        end;
      try
        App := CreateOleObject('Excel.Application');
      except
        MessageDlg('   OLE-',mtError,[mbOk],0);
        exit;
        end;
      App.visible:=false;
      if TemplateFileName='' then App.Workbooks.Add
        else App.Workbooks.Open(TemplateFileName+'.xls');
      DataSet.DisableControls;
      DataSet.First;
      c:=DataSet.FieldCount;
      r:=DataSet.RecordCount;
      rr:=row;
      for j:=1 to r do begin
        for i:=0 to c-1 do
        App.Cells(rr,col+i):=DataSet.Fields[i].AsString;
        p:=trunc((j/r)*100);
        cont:=true;
        DoPercentComplete(p,cont);
        if not cont then
          begin
            App.ActiveWorkBook.Close(false);
            App.Quit;
            App:=unassigned;
            DataSet.EnableControls;
            exit;
          end;//if
        DataSet.Next;
        Inc(rr);
        end;//for j
      if ResultFileName='' then ResultFileName:='result';
      if FileExists(ResultFileName+'.xls') then DeleteFile(ResultFileName+'.xls');
      App.ActiveWorkBook.SaveAs(ResultFileName+'.xls');
      App.ActiveWorkBook.Close(false);
      DataSet.EnableControls;
      App.Quit;
      App:=unassigned;
    end//if
  else
    begin
      if not DataSet.Active then
        begin
          MessageDlg('   ',mtError,[mbOk],0);
          exit;
        end;
      try
        App := CreateOleObject('Word.Application');
      except
        MessageDlg('   OLE-',mtError,[mbOk],0);
        exit;
        end;
      App.visible:=false;
      if TemplateFileName='' then
        begin
          App.ActiveDocument.Close(0);
          App.Quit;
          App:=unassigned;
          DataSet.EnableControls;
          exit;
        end
        else App.Documents.Add(TemplateFileName+'.doc');
      DataSet.DisableControls;
      for i:=0 to DataSet.FieldCount-1 do
        begin
          App.Selection.HomeKey(6);
          App.Selection.Find.ClearFormatting;
          if Length(DataSet.Fields[i].AsString)<250 then
              App.Selection.Find.Execute(FPrefixChar+IntToStr(i+1), false, false, false, false, false,
                                         true, 1, false, DataSet.Fields[i].AsString, 1)
          else begin
            S:=DataSet.Fields[i].AsString;
            while True do begin
              s1:=Copy(s,1,250);
              if Length(S)>250 then s1:=s1+FPrefixChar+IntToStr(i+1);
              App.Selection.Find.Execute(FPrefixChar+IntToStr(i+1), false, false, false, false, false,
                                         true, 1, false, S1, 1);
              if Length(S)<250 then break;
              delete(s,1,250);
            end;//while
          end;//else

          p:=trunc(((i+1)/DataSet.FieldCount)*100);
          cont:=true;
          DoPercentComplete(p,cont);
          if not cont then
            begin
              App.ActiveDocument.Close(0);
              App.Quit;
              App:=unassigned;
              DataSet.EnableControls;
              Exit;
            end;//if
        end;//for i
      if ResultFileName='' then ResultFileName:='result';
      App.ActiveDocument.SaveAs(ResultFileName+'.doc');
      DataSet.EnableControls;
      App.Quit;
      App:=unassigned;
    end;//else
end;//Execute

// Register component classes
procedure Register;
begin
  RegisterComponents('Samples', [TReportMaker]);
end;

initialization
  App:=unassigned;

finalization
  App:=unassigned;

end.
